
#include "cpu.h"
#include  <lib_mem.h>
#include  <os.h>
#include  <bsp.h>
#include  <bsp_os.h>
#include  <bsp_trace.h>
#include  <os_app_hooks.h>

#include  <app_cfg.h>

#include  <em_chip.h>
#include  <em_cmu.h>


static  OS_TCB   StartTaskTCB;
static  OS_TCB   FastLEDTaskTCB;
static  OS_TCB   SlowLEDTaskTCB;

static  CPU_STK  StartTaskStack[START_TASK_STACK_SIZE];
static  CPU_STK  FastLEDTaskStack[FASTLED_TASK_STACK_SIZE];
static  CPU_STK  SlowLEDTaskStack[SLOWLED_TASK_STACK_SIZE];



static  void  StartTask (void  *p_arg);
static  void  FastLEDTask (void  *p_arg);
static  void  SlowLEDTask (void  *p_arg);
static  void  CreateTasks(void);
static  void  CreateObjects (void);


int main(void)
{
	OS_ERR   err;


	CHIP_Init();
	CMU_ClockSelectSet(cmuClock_HF, cmuSelect_HFRCO);

	Mem_Init();                                                 /* Initialize Memory Managment Module                   */
	CPU_IntDis();
	/* Disable all Interrupts.                              */



	OSInit(&err);
#if (TRC_USE_TRACEALYZER_RECORDER==1)
#if (TRC_CFG_RECORDER_MODE == TRC_RECORDER_MODE_SNAPSHOT)
	vTraceEnable(TRC_START);
#else
	vTraceEnable(TRC_INIT);
#endif
#endif


	//App_OS_SetAllHooks();

	OSTaskCreate(&StartTaskTCB,                                 /* Create the start task                                */
			"App Task Start",
			StartTask,
			0u,
			START_TASK_PRIO,
			&StartTaskStack[0u],
			StartTaskStack[START_TASK_STACK_SIZE / 10u],
			START_TASK_STACK_SIZE,
			0u,
			0u,
			0u,
			(OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
			&err);

	OSStart(&err);                                              /* Start multitasking (i.e. give control to uC/OS-III). */

	while (DEF_ON) {                                            /* Should Never Get Here.                               */
		;
	}
}


static  void  StartTask (void *p_arg)
{
	OS_ERR  os_err;

	(void)p_arg;

	BSP_OSTickInit();                                           /* Init uC/OS periodic time src (SysTick).              */
	CPU_Init();                                                 /* Initialize the uC/CPU services                       */

	BSP_LedsInit();
	CreateObjects();                                          /* Create Applicaiton kernel objects                    */
	CreateTasks();                                            /* Create Application tasks                             */
	while (DEF_TRUE)
	{
		BSP_LedToggle(0);
		OSTimeDlyHMSM(0u, 0u, 1u, 0u,OS_OPT_TIME_HMSM_STRICT,&os_err);
	}
}

static  void  FastLEDTask (void *p_arg)
{
	OS_ERR  os_err;
	(void)p_arg;

	while (DEF_TRUE) {
		BSP_LedToggle(1);
		OSTimeDlyHMSM(0u/*h*/,
				0u/*m*/,
				0u/*s*/,
				40u/*ms*/,
				OS_OPT_TIME_HMSM_STRICT,&os_err);
	}
}

static  void  SlowLEDTask (void *p_arg)
{
	OS_ERR  os_err;
	(void)p_arg;

	while (DEF_TRUE) {
		BSP_LedToggle(1);
		OSTimeDlyHMSM(0u, 0u, 0u, 800u,OS_OPT_TIME_HMSM_STRICT,&os_err);
	}
}

static  void  CreateTasks (void)
{
	OS_ERR  err;

	OSTaskCreate(&FastLEDTaskTCB,
			"Fast LED Task",
			FastLEDTask, 0u,
			FASTLED_TASK_PRIO,
			&FastLEDTaskStack[0u],
			FastLEDTaskStack[FASTLED_TASK_STACK_SIZE / 10u],
			FASTLED_TASK_STACK_SIZE,
			0u, 0u, 0u, (OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),&err);
    OSTaskCreate(&SlowLEDTaskTCB,
    		"Slow LED Task",
			SlowLEDTask, 0u,
			SLOWLED_TASK_PRIO,
			&SlowLEDTaskStack[0u],
			SlowLEDTaskStack[SLOWLED_TASK_STACK_SIZE / 10u],
			SLOWLED_TASK_STACK_SIZE,
			0u, 0u, 0u, (OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),&err);
}

static  void  CreateObjects (void)
{
}

