/*******************************************************************************
 * Trace Recorder Library for Tracealyzer v4.0.2
 * Percepio AB, www.percepio.com
 *
 * trcKernelPort.c
 *
 * Terms of Use
 * This file is part of the trace recorder library (RECORDER), which is the 
 * intellectual property of Percepio AB (PERCEPIO) and provided under a
 * license as follows.
 * The RECORDER may be used free of charge for the purpose of recording data
 * intended for analysis in PERCEPIO products. It may not be used or modified
 * for other purposes without explicit permission from PERCEPIO.
 * You may distribute the RECORDER in its original source code form, assuming
 * this text (terms of use, disclaimer, copyright notice) is unchanged. You are
 * allowed to distribute the RECORDER with minor modifications intended for
 * configuration or porting of the RECORDER, e.g., to allow using it on a 
 * specific processor, processor family or with a specific communication
 * interface. Any such modifications should be documented directly below
 * this comment block.  
 *
 * Disclaimer
 * The RECORDER is being delivered to you AS IS and PERCEPIO makes no warranty
 * as to its use or performance. PERCEPIO does not and cannot warrant the 
 * performance or results you may obtain by using the RECORDER or documentation.
 * PERCEPIO make no warranties, express or implied, as to noninfringement of
 * third party rights, merchantability, or fitness for any particular purpose.
 * In no event will PERCEPIO, its technology partners, or distributors be liable
 * to you for any consequential, incidental or special damages, including any
 * lost profits or lost savings, even if a representative of PERCEPIO has been
 * advised of the possibility of such damages, or for any claim by any third
 * party. Some jurisdictions do not allow the exclusion or limitation of
 * incidental, consequential or special damages, or the exclusion of implied
 * warranties or limitations on how long an implied warranty may last, so the
 * above limitations may not apply to you.
 *
 * Tabs are used for indent in this file (1 tab = 4 spaces)
 *
 * Copyright Percepio AB, 2018.
 * www.percepio.com
 ******************************************************************************/

#include <os.h>

#if (defined(TRC_USE_TRACEALYZER_RECORDER) && TRC_USE_TRACEALYZER_RECORDER == 1)

#include <stdint.h>

void* prvTraceGetCurrentTaskHandle(void)
{
	return (void*)OSTCBCurPtr;                                         /* OSTCBCurPtr is a pointer to currently running TCB    */
}

char * prvTraceAppend(const char *p_name, const char *suffix)
{
	static char buf[64];
	
	Str_Copy_N(&buf[0], p_name, 64);
	Str_Cat_N(buf, " ", 64);
	Str_Cat_N(buf, suffix, 64);

	return (&buf[0]);
}

/* Tasks */
uint16_t prvTraceGetTaskNumberLow16(void* handle)
{
	if (handle == 0)
		return 0;
	
	return TRACE_GET_LOW16(((OS_TCB*)handle)->TaskID);
}

uint16_t prvTraceGetTaskNumberHigh16(void* handle)
{
	return TRACE_GET_HIGH16(((OS_TCB*)handle)->TaskID);
}

void prvTraceSetTaskNumberLow16(void* handle, uint16_t value)
{
	((OS_TCB*)handle)->TaskID = TRACE_SET_LOW16(((OS_TCB*)handle)->TaskID, value);
}

void prvTraceSetTaskNumberHigh16(void* handle, uint16_t value)
{
	((OS_TCB*)handle)->TaskID = TRACE_SET_HIGH16(((OS_TCB*)handle)->TaskID, value);
}

/* Task Queues */
#if (OS_CFG_TASK_Q_EN == DEF_ENABLED)
uint16_t prvTraceGetTaskQueueNumberLow16(void* handle)
{
	return TRACE_GET_LOW16(((OS_MSG_Q *)handle)->MsgQID);
}

uint16_t prvTraceGetTaskQueueNumberHigh16(void* handle)
{
	return TRACE_GET_HIGH16(((OS_MSG_Q *)handle)->MsgQID);
}

void prvTraceSetTaskQueueNumberLow16(void* handle, uint16_t value)
{
	((OS_MSG_Q *)handle)->MsgQID = TRACE_SET_LOW16(((OS_MSG_Q *)handle)->MsgQID, value);
}

void prvTraceSetTaskQueueNumberHigh16(void* handle, uint16_t value)
{
	((OS_MSG_Q *)handle)->MsgQID = TRACE_SET_HIGH16(((OS_MSG_Q *)handle)->MsgQID, value);
}
#endif /* (OS_CFG_TASK_Q_EN == DEF_ENABLED) */

/* Task Semaphores */
uint16_t prvTraceGetTaskSemNumberLow16(void* handle)
{
	return TRACE_GET_LOW16(((OS_TCB *)handle)->SemID);
}

uint16_t prvTraceGetTaskSemNumberHigh16(void* handle)
{
	return TRACE_GET_HIGH16(((OS_TCB *)handle)->SemID);
}

void prvTraceSetTaskSemNumberLow16(void* handle, uint16_t value)
{
	((OS_TCB *)handle)->SemID = TRACE_SET_LOW16(((OS_TCB *)handle)->SemID, value);
}

void prvTraceSetTaskSemNumberHigh16(void* handle, uint16_t value)
{
	((OS_TCB *)handle)->SemID = TRACE_SET_HIGH16(((OS_TCB *)handle)->SemID, value);
}

/* Queues */
uint16_t prvTraceGetQueueNumberLow16(void* handle)
{
	return TRACE_GET_LOW16(((OS_Q*)handle)->MsgQ.MsgQID);
}

uint16_t prvTraceGetQueueNumberHigh16(void* handle)
{
	return TRACE_GET_HIGH16(((OS_Q*)handle)->MsgQ.MsgQID);
}

void prvTraceSetQueueNumberLow16(void* handle, uint16_t value)
{
	((OS_Q*)handle)->MsgQ.MsgQID = TRACE_SET_LOW16(((OS_Q*)handle)->MsgQ.MsgQID, value);
}

void prvTraceSetQueueNumberHigh16(void* handle, uint16_t value)
{
	((OS_Q*)handle)->MsgQ.MsgQID = TRACE_SET_HIGH16(((OS_Q*)handle)->MsgQ.MsgQID, value);
}

/* Semaphores */
uint16_t prvTraceGetSemaphoreNumberLow16(void* handle)
{
	return TRACE_GET_LOW16(((OS_SEM*)handle)->SemID);
}

uint16_t prvTraceGetSemaphoreNumberHigh16(void* handle)
{
	return TRACE_GET_HIGH16(((OS_SEM*)handle)->SemID);
}

void prvTraceSetSemaphoreNumberLow16(void* handle, uint16_t value)
{
	((OS_SEM*)handle)->SemID = TRACE_SET_LOW16(((OS_SEM*)handle)->SemID, value);
}

void prvTraceSetSemaphoreNumberHigh16(void* handle, uint16_t value)
{
	((OS_SEM*)handle)->SemID = TRACE_SET_HIGH16(((OS_SEM*)handle)->SemID, value);
}

/* Mutexes */
uint16_t prvTraceGetMutexNumberLow16(void* handle)
{
	return TRACE_GET_LOW16(((OS_MUTEX*)handle)->MutexID);
}

uint16_t prvTraceGetMutexNumberHigh16(void* handle)
{
	return TRACE_GET_HIGH16(((OS_MUTEX*)handle)->MutexID);
}

void prvTraceSetMutexNumberLow16(void* handle, uint16_t value)
{
	((OS_MUTEX*)handle)->MutexID = TRACE_SET_LOW16(((OS_MUTEX*)handle)->MutexID, value);
}

void prvTraceSetMutexNumberHigh16(void* handle, uint16_t value)
{
	((OS_MUTEX*)handle)->MutexID = TRACE_SET_HIGH16(((OS_MUTEX*)handle)->MutexID, value);
}

/* Flags */
uint16_t prvTraceGetFlagNumberLow16(void* handle)
{
	return TRACE_GET_LOW16(((OS_FLAG_GRP*)handle)->FlagID);
}

uint16_t prvTraceGetFlagNumberHigh16(void* handle)
{
	return TRACE_GET_HIGH16(((OS_FLAG_GRP*)handle)->FlagID);
}

void prvTraceSetFlagNumberLow16(void* handle, uint16_t value)
{
	((OS_FLAG_GRP*)handle)->FlagID = TRACE_SET_LOW16(((OS_FLAG_GRP*)handle)->FlagID, value);
}

void prvTraceSetFlagNumberHigh16(void* handle, uint16_t value)
{
	((OS_FLAG_GRP*)handle)->FlagID = TRACE_SET_HIGH16(((OS_FLAG_GRP*)handle)->FlagID, value);
}

/* Mems */
uint16_t prvTraceGetMemNumberLow16(void* handle)
{
	return TRACE_GET_LOW16(((OS_MEM*)handle)->MemID);
}

uint16_t prvTraceGetMemNumberHigh16(void* handle)
{
	return TRACE_GET_HIGH16(((OS_MEM*)handle)->MemID);
}

void prvTraceSetMemNumberLow16(void* handle, uint16_t value)
{
	((OS_MEM*)handle)->MemID = TRACE_SET_LOW16(((OS_MEM*)handle)->MemID, value);
}

void prvTraceSetMemNumberHigh16(void* handle, uint16_t value)
{
	((OS_MEM*)handle)->MemID = TRACE_SET_HIGH16(((OS_MEM*)handle)->MemID, value);
}

#if (TRC_CFG_RECORDER_MODE == TRC_RECORDER_MODE_STREAMING)

/* TzCtrl task Stack and TCB */
static  CPU_STK  TzCtrlStk[TRC_CFG_CTRL_TASK_STACK_SIZE];
static  OS_TCB   TzCtrlTCB;
static  OS_TCB*  ptrTzCtrlTCB = NULL;

static void* pCurrentTCB = NULL;

/* Monitored by TzCtrl task, that give warnings as User Events */
extern volatile uint32_t NoRoomForSymbol;
extern volatile uint32_t NoRoomForObjectData;
extern volatile uint32_t LongestSymbolName;
extern volatile uint32_t MaxBytesTruncated;

/* Keeps track of previous values, to only react on changes. */
static uint32_t NoRoomForSymbol_last = 0;
static uint32_t NoRoomForObjectData_last = 0;
static uint32_t LongestSymbolName_last = 0;
static uint32_t MaxBytesTruncated_last = 0;

/* User Event Channel for giving warnings regarding NoRoomForSymbol etc. */
traceString trcWarningChannel;

TRC_STREAM_PORT_ALLOCATE_FIELDS()

/* Called by TzCtrl task periodically (Normally every 100 ms) */
static void prvCheckRecorderStatus(void);

extern void prvTraceWarning(int errCode);

/* The TzCtrl task - receives commands from Tracealyzer (start/stop) */
static  void     TzCtrl (void  *p_arg);

/*******************************************************************************
 * vTraceEnable
 *
 * Function that enables the tracing and creates the control task. It will halt
 * execution until a Start command has been received if haltUntilStart is true.
 *
 ******************************************************************************/
void vTraceEnable(int startOption)
{
    OS_ERR err;
	int32_t bytes = 0;
	int32_t status;
	extern uint32_t RecorderEnabled;
	TracealyzerCommandType msg;
	
	if (ptrTzCtrlTCB == NULL)
	{
		TRC_STREAM_PORT_INIT();
		trcWarningChannel = xTraceRegisterString("Warnings from Recorder");

  		/* Creates the TzCtrl task - receives trace commands (start, stop, ...) */
		OSTaskCreate((OS_TCB     *)&TzCtrlTCB,             /* Create the control task                              */
        	         (CPU_CHAR   *)"Tracealyzer Control Task",
    	             (OS_TASK_PTR ) TzCtrl,
	                 (void       *) 0,
                	 (OS_PRIO     ) TRC_CFG_CTRL_TASK_PRIORITY,
            	     (CPU_STK    *)&TzCtrlStk[0],
        	         (CPU_STK_SIZE) TRC_CFG_CTRL_TASK_STACK_SIZE / 10u,
    	             (CPU_STK_SIZE) TRC_CFG_CTRL_TASK_STACK_SIZE,
	                 (OS_MSG_QTY  ) 0u,
                	 (OS_TICK     ) 0u,
            	     (void       *) 0,
        	         (OS_OPT      )(OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
    	             (OS_ERR     *)&err);
	
		ptrTzCtrlTCB = &TzCtrlTCB;
	}
	
	if (startOption == TRC_START_AWAIT_HOST)
	{
		/* We keep trying to read commands until the recorder has been started */
		do
		{
			bytes = 0;
			
			status = TRC_STREAM_PORT_READ_DATA(&msg, sizeof(TracealyzerCommandType), (int32_t*)&bytes);
			
			if (status != 0)
			{
				prvTraceWarning(PSF_WARNING_STREAM_PORT_READ);
			}

			if ((status == 0) && (bytes == sizeof(TracealyzerCommandType)))
			{
				if (prvIsValidCommand(&msg))
				{
					if (msg.cmdCode == CMD_SET_ACTIVE && msg.param1 == 1)
					{
						/* On start, init and reset the timestamping */
						TRC_PORT_SPECIFIC_INIT();
					}
					
					prvProcessCommand(&msg);
				}
			}
		}
		while (RecorderEnabled == 0);
	}
	else if (startOption == TRC_START)
	{
		/* We start streaming directly - this assumes that the interface is ready! */
		TRC_PORT_SPECIFIC_INIT();
		
		msg.cmdCode = CMD_SET_ACTIVE;
		msg.param1 = 1;
		prvProcessCommand(&msg);
	}
	else
	{
		/* On TRC_INIT */
		TRC_PORT_SPECIFIC_INIT();
	}
}

/*******************************************************************************
 * prvTraceOnBegin
 *
 * Called on trace begin.
 ******************************************************************************/
void prvTraceOnBegin()
{
	TRC_STREAM_PORT_ON_TRACE_BEGIN();
}

/*******************************************************************************
 * prvTraceOnEnd
 *
 * Called on trace end.
 ******************************************************************************/
void prvTraceOnEnd()
{
	TRC_STREAM_PORT_ON_TRACE_END();
}

/*******************************************************************************
 * prvIsNewTCB
 *
 * Tells if this task is already executing, or if there has been a task-switch.
 * Assumed to be called within a trace hook in kernel context.
 ******************************************************************************/
uint32_t prvIsNewTCB(void* pNewTCB)
{
	if (pCurrentTCB != pNewTCB)
	{
		pCurrentTCB = pNewTCB;
		return 1;
	}
	return 0;
}

/*******************************************************************************
 * prvTraceIsSchedulerSuspended
 *
 * Returns true if the RTOS scheduler currently is disabled, thus preventing any
 * task-switches from occurring. Only called from vTraceStoreISREnd.
 ******************************************************************************/
unsigned char prvTraceIsSchedulerSuspended(void)
{
    return OSRunning != OS_STATE_OS_STOPPED;
}


/*******************************************************************************
 * prvCheckRecorderStatus
 *
 * Called by TzCtrl task periodically (every 100 ms - seems reasonable).
 * Checks a number of diagnostic variables and give warnings as user events,
 * in most cases including a suggested solution.
 ******************************************************************************/
static void prvCheckRecorderStatus(void)
{
	if (NoRoomForSymbol > NoRoomForSymbol_last)
	{
		if (NoRoomForSymbol > 0)
		{
			prvTraceWarning(PSF_WARNING_SYMBOL_TABLE_SLOTS);
		}
		NoRoomForSymbol_last = NoRoomForSymbol;
	}

	if (NoRoomForObjectData > NoRoomForObjectData_last)
	{
		if (NoRoomForObjectData > 0)
		{
			prvTraceWarning(PSF_WARNING_OBJECT_DATA_SLOTS);
		}
		NoRoomForObjectData_last = NoRoomForObjectData;
	}

	if (LongestSymbolName > LongestSymbolName_last)
	{
		if (LongestSymbolName > TRC_CFG_SYMBOL_MAX_LENGTH)
		{
			prvTraceWarning(PSF_WARNING_SYMBOL_MAX_LENGTH);
		}
		LongestSymbolName_last = LongestSymbolName;
	}

	if (MaxBytesTruncated > MaxBytesTruncated_last)
	{
		if (MaxBytesTruncated > 0)
		{
			prvTraceWarning(PSF_WARNING_STRING_TOO_LONG);
		}
		MaxBytesTruncated_last = MaxBytesTruncated;
	}
}

/*******************************************************************************
 * TzCtrl
 *
 * Task for sending the trace data from the internal buffer to the stream 
 * interface (assuming TRC_STREAM_PORT_USE_INTERNAL_BUFFER == 1) and for
 * receiving commands from Tracealyzer. Also does some diagnostics.
 ******************************************************************************/
static void TzCtrl(void  *p_arg)
{
    OS_ERR  err;

	TracealyzerCommandType msg;
	int32_t bytes = 0;
	int32_t status = 0;
	(void)p_arg;

	while (1)
	{
		do
		{
			/* Listen for new commands */
			bytes = 0;
			status = TRC_STREAM_PORT_READ_DATA(&msg, sizeof(TracealyzerCommandType), (int32_t*)&bytes);

			if (status != 0)
			{
				prvTraceWarning(PSF_WARNING_STREAM_PORT_READ);
			}

			if ((status == 0) && (bytes == sizeof(TracealyzerCommandType)))
			{
				if (prvIsValidCommand(&msg))
				{
					prvProcessCommand(&msg); /* Start or Stop currently... */
				}
			}

/* If the internal buffer is disabled, the COMMIT macro instead sends the data directly 
   from the "event functions" (using TRC_STREAM_PORT_WRITE_DATA). */			
#if (TRC_STREAM_PORT_USE_INTERNAL_BUFFER == 1)
			/* If there is a buffer page, this sends it to the streaming interface using TRC_STREAM_PORT_WRITE_DATA. */
			bytes = prvPagedEventBufferTransfer();
#endif			
			
		/* If there was data sent or received (bytes != 0), loop around and repeat, if there is more data to send or receive.
		Otherwise, step out of this loop and sleep for a while. */		
		
		} while (bytes != 0);

		prvCheckRecorderStatus();

		OSTimeDly(TRC_CFG_CTRL_TASK_DELAY, OS_OPT_TIME_DLY, &err);	/* 10ms */
	}
}

#endif /*(TRC_CFG_RECORDER_MODE == TRC_RECORDER_MODE_STREAMING)*/


#if (TRC_CFG_RECORDER_MODE == TRC_RECORDER_MODE_SNAPSHOT)

/* Initialization of the object property table */
void vTraceInitObjectPropertyTable()
{
	RecorderDataPtr->ObjectPropertyTable.NumberOfObjectClasses = TRACE_NCLASSES;
	RecorderDataPtr->ObjectPropertyTable.NumberOfObjectsPerClass[0] = TRC_CFG_NQUEUE;
	RecorderDataPtr->ObjectPropertyTable.NumberOfObjectsPerClass[1] = TRC_CFG_NSEMAPHORE;
	RecorderDataPtr->ObjectPropertyTable.NumberOfObjectsPerClass[2] = TRC_CFG_NMUTEX;
	RecorderDataPtr->ObjectPropertyTable.NumberOfObjectsPerClass[3] = TRC_CFG_NFLAG;
	RecorderDataPtr->ObjectPropertyTable.NumberOfObjectsPerClass[4] = TRC_CFG_NMEM;
	RecorderDataPtr->ObjectPropertyTable.NumberOfObjectsPerClass[5] = TRC_CFG_NTASK_SEM;
	RecorderDataPtr->ObjectPropertyTable.NumberOfObjectsPerClass[6] = TRC_CFG_NTASK_Q;
	RecorderDataPtr->ObjectPropertyTable.NumberOfObjectsPerClass[7] = TRC_CFG_NTASK;
	RecorderDataPtr->ObjectPropertyTable.NumberOfObjectsPerClass[8] = TRC_CFG_NISR;
	RecorderDataPtr->ObjectPropertyTable.NameLengthPerClass[0] = TRC_CFG_NAME_LEN_QUEUE;
	RecorderDataPtr->ObjectPropertyTable.NameLengthPerClass[1] = TRC_CFG_NAME_LEN_SEMAPHORE;
	RecorderDataPtr->ObjectPropertyTable.NameLengthPerClass[2] = TRC_CFG_NAME_LEN_MUTEX;
	RecorderDataPtr->ObjectPropertyTable.NameLengthPerClass[3] = TRC_CFG_NAME_LEN_FLAG;
	RecorderDataPtr->ObjectPropertyTable.NameLengthPerClass[4] = TRC_CFG_NAME_LEN_MEM;
	RecorderDataPtr->ObjectPropertyTable.NameLengthPerClass[5] = TRC_CFG_NAME_LEN_TASK_SEM;
	RecorderDataPtr->ObjectPropertyTable.NameLengthPerClass[6] = TRC_CFG_NAME_LEN_TASK_Q;
	RecorderDataPtr->ObjectPropertyTable.NameLengthPerClass[7] = TRC_CFG_NAME_LEN_TASK;
	RecorderDataPtr->ObjectPropertyTable.NameLengthPerClass[8] = TRC_CFG_NAME_LEN_ISR;
	RecorderDataPtr->ObjectPropertyTable.TotalPropertyBytesPerClass[0] = PropertyTableSizeQueue;
	RecorderDataPtr->ObjectPropertyTable.TotalPropertyBytesPerClass[1] = PropertyTableSizeSemaphore;
	RecorderDataPtr->ObjectPropertyTable.TotalPropertyBytesPerClass[2] = PropertyTableSizeMutex;
	RecorderDataPtr->ObjectPropertyTable.TotalPropertyBytesPerClass[3] = PropertyTableSizeFlag;
	RecorderDataPtr->ObjectPropertyTable.TotalPropertyBytesPerClass[4] = PropertyTableSizeMem;
	RecorderDataPtr->ObjectPropertyTable.TotalPropertyBytesPerClass[5] = PropertyTableSizeTaskSem;
	RecorderDataPtr->ObjectPropertyTable.TotalPropertyBytesPerClass[6] = PropertyTableSizeTaskQ;
	RecorderDataPtr->ObjectPropertyTable.TotalPropertyBytesPerClass[7] = PropertyTableSizeTask;
	RecorderDataPtr->ObjectPropertyTable.TotalPropertyBytesPerClass[8] = PropertyTableSizeISR;
	RecorderDataPtr->ObjectPropertyTable.StartIndexOfClass[0] = StartIndexQueue;
	RecorderDataPtr->ObjectPropertyTable.StartIndexOfClass[1] = StartIndexSemaphore;
	RecorderDataPtr->ObjectPropertyTable.StartIndexOfClass[2] = StartIndexMutex;
	RecorderDataPtr->ObjectPropertyTable.StartIndexOfClass[3] = StartIndexFlag;
	RecorderDataPtr->ObjectPropertyTable.StartIndexOfClass[4] = StartIndexMem;
	RecorderDataPtr->ObjectPropertyTable.StartIndexOfClass[5] = StartIndexTaskSem;
	RecorderDataPtr->ObjectPropertyTable.StartIndexOfClass[6] = StartIndexTaskQ;
	RecorderDataPtr->ObjectPropertyTable.StartIndexOfClass[7] = StartIndexTask;
	RecorderDataPtr->ObjectPropertyTable.StartIndexOfClass[8] = StartIndexISR;
	RecorderDataPtr->ObjectPropertyTable.ObjectPropertyTableSizeInBytes = TRACE_OBJECT_TABLE_SIZE;
}

/* Initialization of the handle mechanism, see e.g, prvTraceGetObjectHandle */
void vTraceInitObjectHandleStack()
{
	objectHandleStacks.indexOfNextAvailableHandle[0] = 0;
	objectHandleStacks.indexOfNextAvailableHandle[1] = TRC_CFG_NQUEUE;
	objectHandleStacks.indexOfNextAvailableHandle[2] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE;
	objectHandleStacks.indexOfNextAvailableHandle[3] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX;
	objectHandleStacks.indexOfNextAvailableHandle[4] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX + TRC_CFG_NFLAG;
	objectHandleStacks.indexOfNextAvailableHandle[5] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX + TRC_CFG_NFLAG + TRC_CFG_NMEM;
	objectHandleStacks.indexOfNextAvailableHandle[6] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX + TRC_CFG_NFLAG + TRC_CFG_NMEM + TRC_CFG_NTASK_SEM;
	objectHandleStacks.indexOfNextAvailableHandle[7] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX + TRC_CFG_NFLAG + TRC_CFG_NMEM + TRC_CFG_NTASK_SEM + TRC_CFG_NTASK_Q;
	objectHandleStacks.indexOfNextAvailableHandle[8] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX + TRC_CFG_NFLAG + TRC_CFG_NMEM + TRC_CFG_NTASK_SEM + TRC_CFG_NTASK_Q + TRC_CFG_NTASK;
	
	objectHandleStacks.lowestIndexOfClass[0] = 0;
	objectHandleStacks.lowestIndexOfClass[1] = TRC_CFG_NQUEUE;
	objectHandleStacks.lowestIndexOfClass[2] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE;
	objectHandleStacks.lowestIndexOfClass[3] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX;
	objectHandleStacks.lowestIndexOfClass[4] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX + TRC_CFG_NFLAG;
	objectHandleStacks.lowestIndexOfClass[5] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX + TRC_CFG_NFLAG + TRC_CFG_NMEM;
	objectHandleStacks.lowestIndexOfClass[6] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX + TRC_CFG_NFLAG + TRC_CFG_NMEM + TRC_CFG_NTASK_SEM;
	objectHandleStacks.lowestIndexOfClass[7] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX + TRC_CFG_NFLAG + TRC_CFG_NMEM + TRC_CFG_NTASK_SEM + TRC_CFG_NTASK_Q;
	objectHandleStacks.lowestIndexOfClass[8] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX + TRC_CFG_NFLAG + TRC_CFG_NMEM + TRC_CFG_NTASK_SEM + TRC_CFG_NTASK_Q + TRC_CFG_NTASK;

	objectHandleStacks.highestIndexOfClass[0] = TRC_CFG_NQUEUE - 1;
	objectHandleStacks.highestIndexOfClass[1] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE - 1;
	objectHandleStacks.highestIndexOfClass[2] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX - 1;
	objectHandleStacks.highestIndexOfClass[3] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX + TRC_CFG_NFLAG - 1;
	objectHandleStacks.highestIndexOfClass[4] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX + TRC_CFG_NFLAG + TRC_CFG_NMEM - 1;
	objectHandleStacks.highestIndexOfClass[5] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX + TRC_CFG_NFLAG + TRC_CFG_NMEM + TRC_CFG_NTASK_SEM - 1;
	objectHandleStacks.highestIndexOfClass[6] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX + TRC_CFG_NFLAG + TRC_CFG_NMEM + TRC_CFG_NTASK_SEM + TRC_CFG_NTASK_Q - 1;
	objectHandleStacks.highestIndexOfClass[7] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX + TRC_CFG_NFLAG + TRC_CFG_NMEM + TRC_CFG_NTASK_SEM + TRC_CFG_NTASK_Q + TRC_CFG_NTASK - 1;
	objectHandleStacks.highestIndexOfClass[8] = TRC_CFG_NQUEUE + TRC_CFG_NSEMAPHORE + TRC_CFG_NMUTEX + TRC_CFG_NFLAG + TRC_CFG_NMEM + TRC_CFG_NTASK_SEM + TRC_CFG_NTASK_Q + TRC_CFG_NTASK + TRC_CFG_NISR - 1;
}

/* Returns the "Not enough handles" error message for this object class */
const char* pszTraceGetErrorNotEnoughHandles(traceObjectClass objectclass)
{
	switch(objectclass)
	{
	case TRACE_CLASS_QUEUE:
		return "Not enough QUEUE handles - increase TRC_CFG_NQUEUE in trcSnapshotConfig.h";
	case TRACE_CLASS_SEMAPHORE:
		return "Not enough SEMAPHORE handles - increase TRC_CFG_NSEMAPHORE in trcSnapshotConfig.h";
	case TRACE_CLASS_MUTEX:
		return "Not enough MUTEX handles - increase TRC_CFG_NMUTEX in trcSnapshotConfig.h";
	case TRACE_CLASS_FLAG:
		return "Not enough FLAG handles - increase TRC_CFG_NFLAG in trcSnapshotConfig.h";
	case TRACE_CLASS_MEM:
		return "Not enough MEM handles - increase TRC_CFG_NMEM in trcSnapshotConfig.h";
	case TRACE_CLASS_TASK_SEM:
		return "Not enough TASK_SEM handles - increase TRC_CFG_NTASK_SEM in trcSnapshotConfig.h";
	case TRACE_CLASS_TASK_Q:
		return "Not enough TASK_Q handles - increase TRC_CFG_NTASK_Q in trcSnapshotConfig.h";
	case TRACE_CLASS_TASK:
		return "Not enough TASK handles - increase TRC_CFG_NTASK in trcSnapshotConfig.h";
	case TRACE_CLASS_ISR:
		return "Not enough ISR handles - increase TRC_CFG_NISR in trcSnapshotConfig.h";
	default:
		return "pszTraceGetErrorHandles: Invalid objectclass!";
	}
}

/*******************************************************************************
 * prvTraceIsSchedulerSuspended
 *
 * Returns true if the RTOS scheduler currently is disabled, thus preventing any
 * task-switches from occurring. Only called from vTraceStoreISREnd.
 ******************************************************************************/
#if (TRC_CFG_INCLUDE_ISR_TRACING == 1)
unsigned char prvTraceIsSchedulerSuspended(void)
{
    return OSRunning != OS_STATE_OS_STOPPED;
}
#endif

#endif /* Snapshot mode */

#endif /*(TRC_USE_TRACEALYZER_RECORDER == 1)*/
